#include "AudioMenu.h"


AudioMenu::AudioMenu (RioAudio *audio, Window *parent, const char *name)
    : Menu (font_helvetica_10, parent, true, name)
{
    typedef void (Menu::*MenuMemberFn)(Line *line);

    this->audio = audio;

    lines.push_back (new Line ("Volume", (MenuMemberFn)&AudioMenu::menuVolume));
    lines.push_back (new Line ("Balance",
                                    (MenuMemberFn)&AudioMenu::menuBalance));
    lines.push_back (new Line ("Treble", (MenuMemberFn)&AudioMenu::menuTreble));
    lines.push_back (new Line ("Bass", (MenuMemberFn)&AudioMenu::menuBass));
    lines.push_back (new Line ("Contrast",
                                    (MenuMemberFn)&AudioMenu::menuContrast));
    lines.push_back (new Line ("Back",   (MenuMemberFn)&AudioMenu::menuBack));
    draw();
}

void
AudioMenu::menuTreble (Line *line)
{
    if (!treble) {
        treble  = new TrebleSlider (audio, this);
    }
    switchTo (treble);
}

void
AudioMenu::menuBass (Line *line)
{
    if (!bass) {
        bass    = new BassSlider (audio, this);
    }
    switchTo (bass);
}

void
AudioMenu::menuBalance (Line *line)
{
    if (!balance) {
        balance = new BalanceSlider (audio, this);
    }
    switchTo (balance);
}

void
AudioMenu::menuContrast (Line *line)
{
    if (!contrast) {
        contrast = new ContrastSlider (audio, this);
    }
    switchTo (contrast);
}

void
AudioMenu::menuBack (Line *line)
{
    switchToParent();
}

void
AudioMenu::volumeWindow (void)
{
    menuVolume (NULL);
}

void 
AudioMenu::menuVolume (Line *line)
{
    if (!volume) {
        volume = new VolumeSlider (audio, this);
    }
    switchTo (volume);
}

void
AudioMenu::randomFunc (Line *line)
{
    cerr << "randomFunc called from " << name() << "::" << line->text << endl;
}
