#include <sys/fcntl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <linux/un.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "EmuDisplay.h"

EmuDisplay::EmuDisplay (const char *dev, int rows, int cols, int memsize)
    : DisplayDev (rows, cols, memsize), powerState (false)
{
    struct sockaddr_un sun;
    fd = socket (PF_UNIX, SOCK_STREAM, 0);
    if (fd  < 0) {
        perror ("socket");
        throw;
    }

    sun.sun_family = AF_UNIX;
    strncpy (sun.sun_path, dev, UNIX_PATH_MAX);
    if (connect (fd, (struct sockaddr *)&sun, sizeof (sun)) < 0) {
        close (fd);
        perror ("connect [display channel] failed");
        throw;
    }

    // Attempt to establish communications channel for the emulated display
    ctl = socket (PF_UNIX, SOCK_STREAM, 0);
    strncpy (sun.sun_path, dev, UNIX_PATH_MAX);
    strncat (sun.sun_path, ".ctl", UNIX_PATH_MAX);
    if (connect (ctl, (struct sockaddr *)&sun, sizeof (sun)) < 0) {
        close (ctl);
        ctl = -1;
        perror ("connect [communication channel] failed");
    }

    mem = (char *)malloc (memsize);
    if (!mem) {
        perror ("malloc");
        throw;
    }
}

EmuDisplay::~EmuDisplay (void)
{
    close (fd);
    free (mem);
}

int
EmuDisplay::sendReadCommand (int cmd)
{
    if (ctl < 0) return -1;

    if (write (ctl, &cmd, sizeof (cmd)) < sizeof (cmd)) {
        perror ("write cmd");
        return -1;
    }

    int val;
    if (read (ctl, &val, sizeof (val)) < sizeof (val)) {
        perror ("read data");
        return -1;
    }
    return val;
}

void
EmuDisplay::sendWriteCommand (int cmd, int val)
{
    if (ctl < 0) return;
    if (write (ctl, &cmd, sizeof (cmd)) < sizeof (cmd)) {
        perror ("write cmd");
        return;
    }

    if (write (ctl, &val, sizeof (val)) < sizeof (val)) {
        perror ("write data");
        return;
    }
}

void
EmuDisplay::setContrast (int lvl)
{
    sendWriteCommand (EMPEG_DISPLAY_SETCONTRAST, lvl);
}

int
EmuDisplay::getContrast (void)
{
    return sendReadCommand (EMPEG_DISPLAY_GETCONTRAST);
}

void
EmuDisplay::setBacklight (int lvl)
{
    sendWriteCommand (EMPEG_DISPLAY_SETBACKLIGHT, lvl);
}

int
EmuDisplay::getBacklight (void)
{
    return sendReadCommand (EMPEG_DISPLAY_GETBACKLIGHT);
}

void
EmuDisplay::setPower (bool on)
{
    powerState = on ? 1 : 0;

    sendWriteCommand (EMPEG_DISPLAY_POWER, powerState);
}

bool
EmuDisplay::getPower (void)
{
    return powerState == 1;
}

void
EmuDisplay::refresh (void)
{
    write (fd, mem, memsize());
}
