#include "RioAudio.h"


#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <unistd.h>
#include <iostream>
#include <sys/soundcard.h>
#include <asm/arch-clps7212/empeg7212.h>

#define AUDIO_DEVICE    "/dev/audio"
#define MIXER_DEVICE    "/dev/mixer"
#define AUDIO_FILLSZ    4608

static  char buffer[AUDIO_FILLSZ];
static  int  fdaudio, fdmixer;

//
// Must output buffers of 4608 bytes or audio driver barfs.  
// Sample rate is always 44.1k, format always 16-bit stereo, alternating 
// left/right samples.
//
static void 
output(void *buf)
{
    int devmask;
    int wrote = write(fdaudio, buf, AUDIO_FILLSZ);
    if (wrote != AUDIO_FILLSZ) {
        printf("Audio write failed: %d.\n", wrote);
        exit(1);
    }
    ioctl(fdmixer, SOUND_MIXER_READ_DEVMASK, &devmask);
}

static void
volume(int vol)
{
}

static void
balance(int bal)
{
    if (ioctl(fdmixer, EMPEG_MIXER_WRITE_BALANCE, &bal) == -1) {
        perror("ioctl(mixer balance)");
        exit(1);
    }
}

static void
soft_mute(int m)
{
    if (ioctl(fdmixer, EMPEG_MIXER_SET_SAM, &m) == -1) {
        perror("ioctl(SAM)");
        exit(1);
    }
}

static void
hard_mute(int m)
{
    if (ioctl(fdmixer, EMPEG_MIXER_SET_HAM, &m) == -1) {
        perror("ioctl(HAM)");
        exit(1);
    }
}
RioAudio::RioAudio (const char *dev)
{
    fdmixer = open (dev, O_RDWR);
    if (fdmixer < 0) {
        cerr << "RioAudio: Failure while opening " << dev << ": "
             << strerror (errno) << endl;
        cerr << "          Mixer functionality unavailable." << endl;
        throw strerror (errno);
    }
    if (ioctl(fdmixer, SOUND_MIXER_READ_DEVMASK, &devmask) < 0) {
        cerr << "RioAudio: Failure to retreive mixer properties for "
             << dev << ": " << strerror (errno) << endl;
        cerr << "          Mixer functionality unavailable." << endl;
        throw strerror (errno);
    }

    _getBalance ();
}

void
RioAudio::_getVolume (int *l, int *r)
{
    int cmd, v = 0;
    if (devmask & SOUND_MASK_PCM)
        cmd = MIXER_READ(SOUND_MIXER_PCM);
    else if (devmask & SOUND_MASK_VOLUME)
        cmd = MIXER_READ(SOUND_MIXER_VOLUME);
    else
        return;

    if (ioctl(fdmixer, cmd, &v) < 0) {
        cerr << "RioAudio::_getVolume - ioctl failed!" << endl;
        *l = -1; *r = -1;
        return;
    }

    *r = (v & 0xFF00) >> 8;
    *l = (v & 0x00FF);
}


void
RioAudio::_setVolume (int l, int r)
{
    int cmd, v, devs;
    if (devmask & SOUND_MASK_PCM)
        cmd = MIXER_WRITE(SOUND_MIXER_PCM);
    else if (devmask & SOUND_MASK_VOLUME)
        cmd = MIXER_WRITE(SOUND_MIXER_VOLUME);
    else
        return;
    v = (r << 8) | l;
    ioctl(fdmixer, cmd, &v);
}

int
RioAudio::getVolume (void)
{
    int l, r;
    _getVolume (&l, &r);

    return (l >= r) ? l : r;
}

void
RioAudio::setVolume (int vol)
{
    int balanced_vol = (vol * (100 - abs(balance))) / 100;
    if (balance < 5)
        _setVolume(vol, balanced_vol);
    else if (balance > 5)
        _setVolume(balanced_vol, vol);
    else
        _setVolume (vol, vol);
}

// Calculates the current balance based on the current volumes
int
RioAudio::_getBalance (void)
{
    int lchan, rchan;
    _getVolume(&lchan, &rchan);

    if (lchan < rchan) /* balance > 0 */
        balance = 100 - ((100 * lchan) / rchan);
    else if (lchan > rchan) /* balance < 0 */
        balance = (100 - ((100 * rchan) / lchan)) * -1;
    else /* balance = 0 */
        balance = 0;
}

void
RioAudio::setBalance (int bal)
{
    balance = bal;
    /* This just adjusts the balance using the current volume */
    setVolume (getVolume());
}

int
RioAudio::getBalance (void)
{
    return balance;
}

void
RioAudio::setTreble (int lev)
{
    if (devmask & SOUND_MASK_TREBLE) {
        ioctl(fdmixer, MIXER_WRITE(SOUND_MIXER_TREBLE), &lev);
    }
}

int
RioAudio::getTreble (void)
{
    int val = -1;
    if (devmask & SOUND_MASK_TREBLE) {
        ioctl(fdmixer, MIXER_READ(SOUND_MIXER_TREBLE), &val);
    }
    return val;

}

void
RioAudio::setBass (int lev)
{
    if (devmask & SOUND_MASK_BASS) {
        ioctl(fdmixer, MIXER_WRITE(SOUND_MIXER_BASS), &lev);
    } else {
        cerr << "Bass ioctl unsupported" << endl;
    }
}

int
RioAudio::getBass (void)
{
    int val = -1;
    if (devmask & SOUND_MASK_BASS) {
        ioctl(fdmixer, MIXER_READ(SOUND_MIXER_BASS), &val);
    } else {
        cerr << "Bass ioctl unsupported" << endl;
    }
    return val;
}

void
RioAudio::setSWMute (bool state)
{
}

bool
RioAudio::getSWMute (void)
{
}

void
RioAudio::setHWMute (bool state)
{
}

bool
RioAudio::getHWMute (void)
{
}

