#include <errno.h>
#include <sys/socket.h>
#include <linux/un.h>
#include <sys/stat.h>
#include <unistd.h>
#include "UnixServerSocket.h"

UnixServerSocket::UnixServerSocket (const char *path, QObject *parent,
                                    const char *name)
    : QServerSocket (parent, name)
{
    int sock;
    struct stat fileinfo;

    if (stat (path, &fileinfo) < 0) {
        if (errno != ENOENT) {
            throw strerror (errno);
        }
    } else {
        if (S_ISSOCK (fileinfo.st_mode)) {
            unlink (path);
        }
    }

    sock = ::socket (PF_UNIX, SOCK_STREAM, 0);
    if (sock < 0) {
        throw strerror (errno);
    }

    struct sockaddr_un sun;
    sun.sun_family = AF_UNIX;
    if (strlen (path) > UNIX_PATH_MAX) {
        close (sock);
        throw "Path too long";
    }
    strncpy (sun.sun_path, path, UNIX_PATH_MAX);
    if (bind (sock, (struct sockaddr *)&sun, sizeof (sun)) < 0) {
        close (sock);
        throw strerror (errno);
    }

    if (listen (sock, 1) < 0) {
        int err = errno;
        shutdown (sock, SHUT_RDWR);
        close (sock);
        throw strerror (err);
    }

    setSocket (sock);
}

UnixServerSocket::~UnixServerSocket()
{
    shutdown (socket(), SHUT_RDWR);
    close (socket());
}

void
UnixServerSocket::newConnection (int sock)
{
    QSocket *qsock = new QSocket();
    qsock->setSocket (sock);
    emit connectionEstablished(sock);
}
