#include <qpainter.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <iostream>
#include <sys/socket.h>
#include <linux/un.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include "XRioScreen.h"

XRioScreen::XRioScreen (int x, int y, QWidget *parent = 0,
                        const char *name = 0, WFlags f = 0)
    : _width (x), _height (y), _memsize (x*y), _scale (1), mem (NULL),
      _contrast (DEFAULT_CONTRAST_VALUE),
      QWidget (parent, name, f)
{
    mem = new char[_memsize];
    setMinimumSize (_width*2, _height*2);
    setBacklightLevel (0);
}


XRioScreen::~XRioScreen()
{
    if (mem) delete mem;
}

void
XRioScreen::update (char *buf, int size)
{
    if (size != 4096) {
        cerr << "Only " << size << " bytes provided :(" << endl;
    } else {
        memcpy (mem, buf, 4096);
        repaint ();
    }
}

void
XRioScreen::paintEvent (QPaintEvent *pe)
{
    if (width() / _width > height() / _height) {
        _scale = height() / _height;
    } else {
        _scale = width() / _width;
    }

    offX = (width() - (_width*_scale))/2;
    offY = (height() - (_height*_scale))/2;

    QPainter screen(this);
    screen.setBrush (Qt::black);
    screen.setPen (Qt::black);
    screen.drawRect (offX - 1, offY - 1, _width*_scale + 2, _height*_scale + 2);
    for (int y = 0; y < _height; y++) {
        for (int x = 0; x < _width; x++) {
            char *src = mem + (_width/2 * y) + (x/2);
            int mask = (x%2) ? 0xf0 : 0x0f;
            if (*src & mask) {
                screen.setBrush (fgColor);
                screen.setPen (fgColor);
            } else {
                screen.setBrush (bgColor);
                screen.setPen (bgColor);
            }
            drawScaledPixel (screen, _scale, x, y);
        }
    }
}

void
XRioScreen::drawScaledPixel (QPainter &screen, int _scale, int x, int y)
{
    screen.fillRect (offX + (x*_scale), offY + (y*_scale), _scale, _scale,
                     screen.brush());
}

int
XRioScreen::contrastTransform (QColor &color, int lvl)
{
    int h, s, v;
    int new_v;
    color.hsv (&h, &s, &v);

    if (lvl > 10) { // getting ligher, v increases
        int incr = (255 - v) / 10;
        new_v = v + incr * (lvl - 10);
    } else {            // getting darker, v decreases
        int incr = v / 10;
        new_v = incr * lvl;
    }

    return new_v ;
}

void
XRioScreen::setContrastLevel (int lvl)
{
    int userval = 20 - USER_CONTRAST_VALUE (lvl);
    int h, s, v;

    cbgColor.hsv (&h, &s, &v);
    v = contrastTransform (cbgColor, userval);
    bgColor.setHsv (h, s, v);

    cfgColor.hsv (&h, &s, &v);
    v = contrastTransform (cfgColor, userval);
    fgColor.setHsv (h, s, v);
    repaint ();

    _contrast = lvl;
}

int
XRioScreen::getContrastLevel (void)
{
    cerr << "Returning " << _contrast << endl;
    return _contrast;
}

void
XRioScreen::setBacklightLevel (int lvl)
{
    /* Give everything a light blue hue if the backlight is on */
    if (lvl > 0) {
        _backlight = 1;
        fgColor.setNamedColor (FGCOLOR_BACKLIGHT_ON);
        bgColor.setNamedColor (BGCOLOR_BACKLIGHT_ON);
    } else {
        _backlight = 0;
        fgColor.setNamedColor (FGCOLOR_BACKLIGHT_OFF);
        bgColor.setNamedColor (BGCOLOR_BACKLIGHT_OFF);
    }
    cfgColor = fgColor;
    cbgColor = bgColor;

    setContrastLevel (_contrast);
    repaint ();
}

int
XRioScreen::getBacklightLevel (void)
{
    return _backlight;
}

void
XRioScreen::setPower (bool state)
{
    cerr << "Turning " << (state ? "on" : "off") << " display power." << endl;
}

