#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <linux/reboot.h>
#include <pthread.h>
#include <sys/socket.h>
#include <linux/un.h>
#include <sys/ioctl.h>
#include "ir.h"


#include "RootWindow.h"


extern "C" {
    int setup_console (void);
};
void input_loop (char *device);

int
main()
{
    RootWindow *window;

    try {
        window = new RootWindow (NULL);
    } catch (int err) {
        fprintf (stderr, "Error while creating root window: %s\n",
                 strerror (-err));
    }
    
#ifndef USE_EMULATED_SYSTEM
    setup_console ();
#endif


#ifdef USE_EMULATED_SYSTEM
    input_loop ("irport");
#else
    input_loop ("/dev/ir");
#endif
}

void reinitialize_unit();

void
input_loop (char *device)
{
    int fd;
    uint32_t ircode;
    int rc;
    int power_state = 1;
    int cmd_count = 0;
#ifdef USE_EMULATED_SYSTEM
    struct sockaddr_un sun;
#endif


#ifndef USE_EMULATED_SYSTEM
    fd = open (device, O_RDONLY);
    if (fd < 0) {
        perror ("open");
        return;
    }
    
    /* Wait 1/2 second before sending repeats */
    if (ioctl (fd, EMPEG_IR_WRITE_RPTDELAY, 500000) < 0) {
        perror ("ioctl");
        close (fd);
        return;
    }
#else
    fd = socket (PF_UNIX, SOCK_STREAM, 0);
    if (fd < 0) {
        perror ("socket");
        return;
    }

    sun.sun_family = AF_UNIX;
    strncpy (sun.sun_path, device, UNIX_PATH_MAX);
    if (connect (fd, (struct sockaddr *)&sun, sizeof (sun)) < 0) {
        perror ("connect");
        close (fd);
        return;
    }
#endif

    while ( (rc = read (fd, (char *)&ircode, sizeof (ircode))) > 0) {
        if (++cmd_count < 4) continue;
        if (power_state) {
            bool again = ircode & 0x8000000;
            switch (ircode & 0x0fffffff) {
            case PANEL_POWER_DN:
                Window::onPowerButton (again);
                fprintf (stderr, "(panel power button down)\n");
                break;
            case PANEL_POWER_UP:
                fprintf (stderr, "(panel power button up)\n");
                exit (0);
                break;
            case PANEL_REPEAT_DN:
                Window::onRepeatButton (again);
                fprintf (stderr, "(panel repeat button up)\n");
                break;
            case PANEL_REPEAT_UP:
                fprintf (stderr, "(panel repeat button down)\n");
                break;
            case PANEL_RANDOM_DN:
                Window::onRandomButton (again);
                break;
            case PANEL_RANDOM_UP:
                fprintf (stderr, "(panel random button up)\n");
                break;
            case PANEL_MENU_DN:
                Window::onMenuButton (again);
                break;
            case PANEL_MENU_UP:
                fprintf (stderr, "(panel menu button up)\n");
                break;
            case PANEL_KNOB_BTN_DN:
                fprintf (stderr, "(panel knob button down)\n");
                break;
            case PANEL_KNOB_BTN_UP:
                Window::onKnobButton (again);
                break;
            case PANEL_KNOB_RIGHT:
                Window::onKnobRight (again);
                break;
            case PANEL_KNOB_LEFT:
                Window::onKnobLeft (again);
                break;
            case PANEL_PLAY_DN:
                Window::onPlayButton (again);
                break;
            case PANEL_PLAY_UP:
                fprintf (stderr, "(panel play button up)\n");
                break;
            case PANEL_REW_DN:
                Window::onRewButton (again);
                break;
            case PANEL_REW_UP:
                fprintf (stderr, "(panel rewind button up)\n");
                break;
            case PANEL_FF_DN:
                Window::onFFButton (again);
                break;
            case PANEL_FF_UP:
                fprintf (stderr, "(panel forward button up)\n");
                break;
            case PANEL_STOP_DN:
                Window::onStopButton (again);
                break;
            case PANEL_STOP_UP:
                fprintf (stderr, "(panel stop button up)\n");
                break;
            case REMOTE_POWER:
                if ((ircode & 0x80000000) == 0) {
                    fprintf (stderr, "(remote power pushed - powering off display)\n");
                    power_state = 0;
                    Window::setPower (false);
                }
                break;
            case REMOTE_MENU:
                fprintf (stderr, "(remote menu pushed)\n");
                break;
            case REMOTE_LIST:
                fprintf (stderr, "(remote list pushed)\n");
                break;
            case REMOTE_SEARCH:
                fprintf (stderr, "(remote search pushed)\n");
                break;
            case REMOTE_SELECT_UP_ARROW:
                fprintf (stderr, "(remote select up pushed)\n");
                break;
            case REMOTE_SELECT_DN_ARROW:
                fprintf (stderr, "(remote select down pushed)\n");
                break;
            case REMOTE_ENTER:
                fprintf (stderr, "(remote enter pushed)\n");
                break;
            case REMOTE_CANCEL:
                fprintf (stderr, "(remote cancel pushed)\n");
                break;
            case REMOTE_MUTE:
                fprintf (stderr, "(remote mute pushed)\n");
                break;
            case REMOTE_VOLUME_UP_ARROW:
                fprintf (stderr, "(remote volume up pushed)\n");
                break;
            case REMOTE_VOLUME_DN_ARROW:
                fprintf (stderr, "(remote volume down pushed)\n");
                break;
            case REMOTE_RANDOM:
                fprintf (stderr, "(remote random pushed)\n");
                break;
            case REMOTE_REPEAT:
                fprintf (stderr, "(remote repeat pushed)\n");
                break;
            case REMOTE_PLAY_PAUSE:
                fprintf (stderr, "(remote play/pause pushed)\n");
                break;
            case REMOTE_BACK:
                fprintf (stderr, "(remote back pushed)\n");
                break;
            case REMOTE_FORW:
                fprintf (stderr, "(remote forward pushed)\n");
                break;
            case REMOTE_STOP:
                fprintf (stderr, "(remote stop pushed)\n");
                break;
            case REMOTE_1:
                fprintf (stderr, "(remote 1 pushed)\n");
                break;
            case REMOTE_2:
                fprintf (stderr, "(remote 2 pushed)\n");
                break;
            case REMOTE_3:
                fprintf (stderr, "(remote 3 pushed)\n");
                break;
            case REMOTE_4:
                fprintf (stderr, "(remote 4 pushed)\n");
                break;
            case REMOTE_5:
                fprintf (stderr, "(remote 5 pushed)\n");
                break;
            case REMOTE_6:
                fprintf (stderr, "(remote 6 pushed)\n");
                break;
            case REMOTE_7:
                fprintf (stderr, "(remote 7 pushed)\n");
                break;
            case REMOTE_8:
                fprintf (stderr, "(remote 8 pushed)\n");
                break;
            case REMOTE_9:
                fprintf (stderr, "(remote 9 pushed)\n");
                break;
            case REMOTE_0:
                fprintf (stderr, "(remote 0 pushed)\n");
                break;
            default:
                fprintf (stderr, "Unknown button push with code 0x%08x (ignoring)\n",
                        ircode);
            };
        } else if (ircode == REMOTE_POWER) {
            fprintf (stderr, "(remote power pushed - powering on display)\n");
            power_state = 1;
            Window::setPower (true);
            reinitialize_unit();
        }

    }

    close (fd);
}

void
reinitialize_unit()
{
    printf ("Re-initializing unit...\n");
}
